<?php

namespace Cas\Subcategory\Providers;

use Illuminate\Support\ServiceProvider;

class SubcategoryServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'subcategory');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'subcategory');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerSubcategory();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerSubcategory()
    {
        $this->app->bind('subcategory', function($app) {
            return new Subcategory($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Subcategory', 'Lavalite\Subcategory\Facades\Subcategory');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('cas.subcategory', function ($app) {
            return $this->app->make('Cas\Subcategory\Subcategory');
        });

                // Bind Subcategory to repository
        $this->app->bind(
            'Cas\Subcategory\Interfaces\SubcategoryRepositoryInterface',
            \Cas\Subcategory\Repositories\Eloquent\SubcategoryRepository::class
        );

        $this->app->register(\Cas\Subcategory\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Cas\Subcategory\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and subcategory's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/subcategory.php', 'subcategory'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeSubcategory::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['cas.subcategory'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('cas/subcategory.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/subcategory')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/subcategory')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
